<?php
/**
 * Moony - the PHP simplistic framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2006 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

define('MOONY_VERSION', '0.13.0');
define('MOONY_ROOT_DIR', dirname(__FILE__));

require_once MOONY_ROOT_DIR . '/Moony/Processor.php';

/**
 * Moonyのフロントエンドクラスです。
 * Moonyの動作設定を保持します。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony
{
    /**
     * アクションクラスを格納するディレクトリ
     * @var string
     */
    var $action_dir = 'actions';

    /**
     * テンプレートファイルを格納するディレクトリ
     * @var string
     */
    var $template_dir = 'templates';

    /**
     * キャッシュファイルを格納するディレクトリ
     * @var string
     */
    var $cache_dir;

    /**
     * 入力エンコーディング
     * @var string
     */
    var $input_encoding;

    /**
     * 出力エンコーディング
     * @var string
     */
    var $output_encoding;

    /**
     * 内部エンコーディング
     * @var string
     */
    var $internal_encoding;

    /**
     * 出力バッファリングのハンドラ
     * @var callback
     */
    var $ob_handler;

    /**
     * セッションを自動的に開始するかどうか
     * @var bool
     */
    var $session_auto_start = true;

    /**
     * セッションのオプション設定の連想配列
     * @var array
     */
    var $session_options;

    /**
     * テンプレートエンジンとしてSmartyを使用するかどうか
     * @var bool
     */
    var $use_smarty = false;

    /**
     * Smartyのインスタンスに設定するプロパティ
     * @var array
     */
    var $smarty_properties;

    /**
     * 処理を開始します。
     *
     * @access public
     */
    function start()
    {
        if (is_null($this->internal_encoding)) {
            // internal encoding
            $this->internal_encoding = mb_internal_encoding();
        }
        // check
        $this->_toFullPath($this->action_dir);
        $this->_toFullPath($this->template_dir);
        if ($this->use_smarty) {
            if (!class_exists('Smarty')) {
                // not loaded: Smarty
                trigger_error('Not loaded: Smarty', E_USER_ERROR);
                exit;
            }
        }
        // start
        $processor =& new Moony_Processor();
        $processor->process($this);
    }

    /**
     * アクションクラスを格納するディレクトリを設定します。
     * 初期設定値は「actions」です。
     *
     * @access public
     * @param string $action_dir アクションクラスを格納するディレクトリ
     */
    function setActionDir($action_dir)
    {
        $this->_toFullPath($action_dir);
        $this->action_dir = $action_dir;
    }

    /**
     * テンプレートファイルを格納するディレクトリを設定します。
     * 初期設定値は「templates」です。
     *
     * @access public
     * @param string $template_dir テンプレートファイルを格納するディレクトリ
     */
    function setTemplateDir($template_dir)
    {
        $this->_toFullPath($template_dir);
        $this->template_dir = $template_dir;
    }

    /**
     * キャッシュファイルを格納するディレクトリを設定します。
     * 設定した場合、Moony_Cacheクラスがロードされ、
     * アクションクラス内で使用できるようになります。
     * 設定されない場合、Moony_Cacheクラスはロードされません。
     *
     * @access public
     * @param string $cache_dir キャッシュファイルを格納するディレクトリ
     */
    function setCacheDir($cache_dir)
    {
        $this->cache_dir = $cache_dir;
    }

    /**
     * 入力エンコーディングを設定します。
     * 設定した場合、リクエストパラメータの値は
     * 内部エンコーディングに変換されます。
     *
     * @access public
     * @param string $input_encoding 入力エンコーディング
     */
    function setInputEncoding($input_encoding)
    {
        $this->input_encoding = $input_encoding;
    }

    /**
     * 出力エンコーディングを設定します。
     * 設定した場合、出力時のデフォルトエンコーディングとして使用されます。
     *
     * @access public
     * @param string $output_encoding 出力エンコーディング
     */
    function setOutputEncoding($output_encoding)
    {
        $this->output_encoding = $output_encoding;
    }

    /**
     * 内部エンコーディングを設定します。
     * 設定されない場合、mb_internal_encodingの戻り値を
     * 内部エンコーディングとして使用します。
     *
     * @access public
     * @param string $internal_encoding 内部エンコーディング
     */
    function setInternalEncoding($internal_encoding)
    {
        $this->internal_encoding = $internal_encoding;
    }

    /**
     * 出力バッファリングのハンドラを設定します。
     * 設定した場合、アクションクラスのconvert、validate、executeの
     * 各メソッドが実行される間の
     * 出力バッファリングのハンドラとして用いられます。
     *
     * @access public
     * @param callbacl $ob_handler 出力バッファリングのハンドラ
     */
    function setObHandler($ob_handler)
    {
        $this->ob_handler = $ob_handler;
    }

    /**
     * セッションのオプション設定を設定します。
     * オプションには以下の指定が可能です:
     *   - name: セッション名
     *   - id: セッションID
     *   - save_path: セッションデータの保存先
     *   - cache_expire: カレントのキャッシュの有効期限
     *   - cache_limiter: カレントのキャッシュリミッタ
     *
     * @access public
     * @param array $options セッションのオプション設定
     */
    function setSessionOptions($options)
    {
        $this->session_options = $options;
    }

    /**
     * セッションを自動的に開始するかどうかを設定します。
     * 初期設定値は「true」で、変更しない限り
     * 自動的にセッションが開始されます。
     *
     * @access public
     * @param bool $session_auto_start セッションを自動で開始するかどうか
     */
    function setSessionAutoStart($session_auto_start)
    {
        $this->session_auto_start = (bool) $session_auto_start;
    }

    /**
     * テンプレートエンジンとしてSmartyを使用します。
     * $propertiesに設定された値は、
     * Smartyのインスタンスのプロパティとして設定されます。
     * （例: 'compile_dir' => 'templates_c'）
     * 設定値は既存のプロパティ値を完全に置き換える点に注意してください。
     *
     * @access public
     * @param array $properties Smartyのインスタンスに設定するプロパティ
     */
    function useSmarty($properties)
    {
        $this->use_smarty = true;
        $this->smarty_properties = $properties;
    }

    /**
     * パスをフルパスに変換します。
     * パスが設定されていない場合、
     * または存在しないパスの場合はnullに設定されます。
     *
     * @access protected
     * @param string $path パス文字列
     */
    function _toFullPath(&$path)
    {
        if (strlen(trim($path)) == 0) {
            $path = null;
            return;
        }
        $path = realpath($path);
        if ($path === false) {
            $path = null;
            return;
        }
    }

}
?>
