<?php
/**
 * Moony - the tiny web application framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2006 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

require_once MOONY_ROOT_DIR . '/Moony/Action.php';
require_once MOONY_ROOT_DIR . '/Moony/Registry.php';
require_once MOONY_ROOT_DIR . '/Moony/Request.php';
require_once MOONY_ROOT_DIR . '/Moony/Router.php';
require_once MOONY_ROOT_DIR . '/Moony/Session.php';

/**
 * 処理を順番に行います。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Processor
{
    /**
     * 処理を以下の順番で行います。
     *   1. リクエスト、セッションを司るコンポーネントの生成
     *   2. ルーティング処理（Moony_Router）
     *   3. アクションクラスが見つからない場合、テンプレートを直接表示
     *   4. アクションクラスのメンバ変数にリクエストパラメータを展開
     *   5. アクションクラスのメソッドを順番に実行
     *
     * @access public
     * @param object $moony Moonyのインスタンス（動作設定）
     */
    function process(&$moony)
    {
        // request
        $request =& new Moony_Request();
        if (!ini_get('mbstring.encoding_translation') &&
            !is_null($moony->input_encoding)) {
            // convert encoding
            $request->convertEncoding(
                $moony->input_encoding, $moony->internal_encoding);
        }

        // session
        $session =& new Moony_Session();

        // routing
        if (is_null($moony->action_dir)) {
            // not defined: action_dir
            trigger_error('Not defined: action_dir', E_USER_ERROR);
            exit;
        }
        $router =& new Moony_Router();
        $action = $router->route($request, $moony->action_dir);

        if (is_null($action)) {
            // not found: action class
            // try to render a template directly
            $action =& $this->_createBlankAction();
        }

        // set to the action instance
        $action->request = &$request;
        $action->session = &$session;
        if (!is_null($moony->cache_dir)) {
            // cache
            include_once MOONY_ROOT_DIR . '/Moony/Cache.php';
            $action->cache =& new Moony_Cache($moony->cache_dir);
        }
        if ($moony->use_smarty) {
            // smarty
            include_once MOONY_ROOT_DIR . '/Moony/Smarty.php';
            $action->smarty =& new Moony_Smarty($moony->smarty_properties);
        }

        // registry
        Moony_Registry::set('_moony', $moony);
        Moony_Registry::set('_moony_action', $action);

        $this->_extractParams($action, $request);
        $this->_processAction($action, $moony);
    }

    /**
     * Moony_Actionを継承しただけのアクションクラスを生成、
     * インスタンスを取得します。
     *
     * @access protected
     * @return object アクションクラスのインスタンス
     */
    function &_createBlankAction()
    {
        $code = 'class BlankAction extends Moony_Action {}';
        eval($code);
        $action =& new BlankAction();
        return $action;
    }

    /**
     * リクエストパラメータを
     * アクションクラスのメンバ変数として展開します。
     * 名称が「_」で始まるパラメータと、
     * 「request」「session」「cache」「smarty」という名称のパラメータは
     * 展開されません（Moonyの処理で使用するため）。
     *
     * @access protected
     * @param object $action アクションクラスのインスタンス
     * @param object $request Moony_Requestのインスタンス
     */
    function _extractParams(&$action, &$request)
    {
        $params = $request->getAll();
        $prop_keys = array_keys(get_object_vars($action));
        $ignore_keys = array('request', 'session', 'cache', 'smarty');

        foreach ($params as $name => $value) {
            if ($name[0] != '_' && !in_array($name, $ignore_keys) &&
                in_array($name, $prop_keys)) {
                $action->{$name} = $value;
            }
        }
    }

    /**
     * アクションクラスの処理を実行します。
     * メソッドを以下の順番で実行します。
     * 該当するメソッドが存在しない場合、処理されません。
     *   1. prepareメソッド
     *     - 処理前に必要な準備処理を記述します
     *     - リクエストパラメータの変換処理など
     *   2. convertメソッド
     *     - リクエストパラメータの変換処理など
     *     - 引数としてMoony_Converterのインスタンスが渡されます
     *   3. validateメソッド
     *     - リクエストパラメータの検証処理を記述します
     *     - 引数としてMoony_Validatorのインスタンスが渡されます
     *   4. executeメソッド
     *     - 実際の処理を記述します
     * なお、prepareメソッドの実行後に出力バッファリングが開始されます。
     * また、設定によってはセッションが同じタイミングで開始されます。
     *
     * @access protected
     * @param object $action アクションクラスのインスタンス
     * @param object $moony Moonyのインスタンス（動作設定）
     */
    function _processAction(&$action, &$moony)
    {
        // action::prepare
        if (method_exists($action, 'prepare')) {
            $action->prepare();
        }

        // ob start
        if (is_callable($moony->ob_handler)) {
            // registered handler
            ob_start($moony->ob_handler);
        } else {
            ob_start();
        }

        // session
        if (is_array($moony->session_options)) {
            // set options
            $this->_setSessionOptions(
                $action->session, $moony->session_options);
        }
        if ($moony->session_auto_start) {
            // start automatically
            $action->session->start();
        }

        // action::convert
        if (method_exists($action, 'convert')) {
            include_once MOONY_ROOT_DIR . '/Moony/Converter.php';
            $converter =& new Moony_Converter($moony->internal_encoding);
            $action->convert($converter);
        }

        // action::validate
        if (method_exists($action, 'validate')) {
            include_once MOONY_ROOT_DIR . '/Moony/Validator.php';
            $validator =& new Moony_Validator($moony->internal_encoding);
            $action->validate($validator);
        }

        // action::execute
        $action->execute();

        // ob end
        ob_end_flush();
    }

    /**
     * セッションに関するオプションを設定します。
     *
     * @access protected
     * @param object $session Moony_Sessionのインスタンス
     * @param array $session_options セッションのオプション設定
     */
    function _setSessionOptions(&$session, $session_options)
    {
        foreach ($session_options as $name => $value) {
            switch ($name) {
                case 'name':
                    $session->setName($value);
                    break;
                case 'id':
                    $session->setId($value);
                    break;
                case 'save_path':
                    $session->setSavePath($value);
                    break;
                case 'cache_expire':
                    $session->setCacheExpire($value);
                    break;
                case 'cache_limiter':
                    $session->setCacheLimiter($value);
                    break;
            }
        }
    }
}
?>
