<?php
/**
 * Moony - the tiny web application framework
 *
 * @package   Moony
 * @author    YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @link      http://moony.sourceforge.jp/
 * @copyright 2005-2006 YAMAOKA Hiroyuki
 * @license   http://opensource.org/licenses/bsd-license.php The BSD License
 */

/**
 * リクエスト情報を扱うクラスです。
 *
 * @package Moony
 * @author YAMAOKA Hiroyuki <yamaoka@catwalker.jp>
 * @access public
 */
class Moony_Request
{
    /**
     * リクエストパラメータの連想配列
     * @var array
     */
    var $_params;

    /**
     * アップロードファイルに関する情報の連想配列
     * @var array
     */
    var $_files;

    /**
     * クッキーの値の連想配列
     * @var string
     */
    var $_cookie;

    /**
     * クエリーストリング
     * @var string
     */
    var $_query_string;

    /**
     * PATH_INFO
     * @var string
     */
    var $_path_info;

    /**
     * アクションパス
     * @var string
     */
    var $_action_path;

    /**
     * コンストラクタです。
     *
     * @access public
     */
    function Moony_Request()
    {
        $this->_params = $this->_filter($_POST + $_GET);
        $this->_files = $this->_filter($_FILES);
        $this->_cookie = $this->_filter($_COOKIE);

        // query string
        $query_string = $this->getServer('QUERY_STRING');
        $this->_query_string = urldecode($query_string);

        // path_info
        $path_info_key = isset(
            $_SERVER['ORIG_PATH_INFO']) ? 'ORIG_PATH_INFO' : 'PATH_INFO';
        $path_info = $this->getServer($path_info_key);
        $this->_path_info = urldecode($path_info);
    }

    /**
     * リクエストパラメータの値を取得します。
     * 該当する値が存在しない場合、$defaultを返します。
     *
     * @access public
     * @param string $name リクエストパラメータの名称
     * @param mixed $default 値が存在しない場合のデフォルト値
     * @return string|array リクエストパラメータの値
     */
    function get($name, $default = null)
    {
        if ($this->exists($name)) {
            return $this->_params[$name];
        }
        return $default;
    }

    /**
     * 全てのリクエストパラメータの値を連想配列として取得します。
     *
     * @access public
     * @return array 全てのリクエストパラメータの値の連想配列
     */
    function getAll()
    {
        return $this->_params;
    }

    /**
     * リクエストパラメータの存在有無を調べます。
     *
     * @access public
     * @param string $name リクエストパラメータの名称
     * @return bool リクエストパラメータの存在有無
     */
    function exists($name)
    {
        return isset($this->_params[$name]);
    }

    /**
     * アップロードファイルに関する情報を取得します。
     * $_FILESの1要素を取得します。
     *
     * @access public
     * @param string $name アップロードファイル名
     * @return array アップロードファイルに関する情報の連想配列
     */
    function getFile($name)
    {
        if (isset($this->_files[$name])) {
            return $this->_files[$name];
        }
    }

    /**
     * クッキーの値を取得します。$_COOKIEの1要素を取得します。
     *
     * @access public
     * @param string $name クッキーの名称
     * @return mixed クッキーに保存されていた値
     */
    function getCookie($name)
    {
        if (isset($this->_cookie[$name])) {
            return $this->_cookie[$name];
        }
    }

    /**
     * クエリーストリングを取得します。
     *
     * @access public
     * @return string クエリーストリング
     */
    function getQueryString()
    {
        return $this->_query_string;
    }

    /**
     * PATH_INFOを取得します。
     *
     * @access public
     * @return string PATH_INFO
     */
    function getPathInfo()
    {
        return $this->_path_info;
    }

    /**
     * サーバのURLを取得します。
     * リクエストURLが「http://example.com/test/index.php/foo」の場合、
     * 「http://example.com」を返します。
     *
     * @access public
     * @return string サーバのURL
     */
    function getHostUrl()
    {
        $is_secure = $this->isSecure();
        $host_url = $is_secure ? 'https://' : 'http://';

        // host name
        $host_name = '';
        foreach (array(
            'HTTP_X_FORWARDED_HOST',
            'HTTP_X_FORWARDED_SERVER',
            'HTTP_HOST',
            'SERVER_NAME'
        ) as $name) {
            if (isset($_SERVER[$name])) {
                $host_name = $this->getServer($name);
                break;
            }
        }
        if (($delim_pos = strpos($host_name, ':')) !== false) {
            // remove port
            $host_name = substr($host_name, 0, $delim_pos);
        }

        // port
        $port = null;
        foreach (array(
            'HTTP_X_FORWARDED_PORT',
            'SERVER_PORT'
        ) as $name) {
            if (isset($_SERVER[$name])) {
                $port = $this->getServer($name);
                if ($is_secure && $port == 443) {
                    $port = null;
                } else if (!$is_secure && $port == 80) {
                    $port = null;
                }
                break;
            }
        }

        $host_url .= strtolower($host_name);
        if (!is_null($port)) {
            // append port number
            $host_url .= ':' . $port;
        }

        return $host_url;
    }

    /**
     * 環境変数を取得します。
     * null文字が含まれている場合、除去します。
     *
     * @access public
     * @param string $name 環境変数の名称
     * @return string 環境変数の値
     */
    function getServer($name)
    {
        if (isset($_SERVER[$name])) {
            return str_replace("\0", '', $_SERVER[$name]);
        }
    }

    /**
     * 環境変数に格納されているリクエストヘッダを取得します。
     *
     * @access public
     * @param string $name リクエストヘッダの名称
     * @return string リクエストヘッダの値
     */
    function getHeader($name)
    {
        $server_name = 'HTTP_' . strtr(strtoupper($name), '-', '_');
        return $this->getServer($server_name);
    }

    /**
     * リクエストメソッドがPOSTかどうかを調べます。
     *
     * @access public
     * @return bool リクエストメソッドがPOSTかどうか
     */
    function isPost()
    {
        $method = $this->getServer('REQUEST_METHOD');
        return strtolower($method) == 'post';
    }

    /**
     * リクエストメソッドがGETかどうかを調べます。
     *
     * @access public
     * @return bool リクエストメソッドがGETかどうか
     */
    function isGet()
    {
        $method = $this->getServer('REQUEST_METHOD');
        return strtolower($method) == 'get';
    }

    /**
     * リクエストメソッドがPUTかどうかを調べます。
     *
     * @access public
     * @return bool リクエストメソッドがPUTかどうか
     */
    function isPut()
    {
        $method = $this->getServer('REQUEST_METHOD');
        return strtolower($method) == 'put';
    }

    /**
     * リクエストメソッドがDELETEかどうかを調べます。
     *
     * @access public
     * @return bool リクエストメソッドがDELETEかどうか
     */
    function isDelete()
    {
        $method = $this->getServer('REQUEST_METHOD');
        return strtolower($method) == 'delete';
    }

    /**
     * XMLHttpRequestを用いたアクセスかどうかを調べます。
     * Prototype.jsが付加する"X-Requested-With"ヘッダを利用して判定します。
     *
     * @access public
     * @return bool XMLHttpRequestを用いたアクセスかどうか
     */
    function isXmlHttpRequest()
    {
        $requested_with = $this->getHeader('X-Requested-With');
        return $requested_with == 'XMLHttpRequest';
    }

    /**
     * isXmlHttpRequestメソッドのエイリアスです。
     *
     * @access public
     * @return bool XMLHttpRequestを用いたアクセスかどうか
     */
    function isXhr()
    {
        return $this->isXmlHttpRequest();
    }

    /**
     * SSLを用いた通信かどうかを調べます。
     *
     * @access public
     * @return bool SSLを用いた通信かどうか
     */
    function isSecure()
    {
        return strtolower($this->getServer('HTTPS')) === 'on';
    }

    /**
     * アクションにPATH_INFOで渡されたパラメータを取得します。
     * $indexが設定されていない場合、文字列全てを返します。
     * $indexが設定されている場合、
     * PATH_INFOを「/」で区切ったインデックスに従った値を返します。
     * 例えば、「/foo/bar/baz/1」にアクセスがあり、
     * アクション名が「Foo」の場合、以下のようになります。
     *   - param()の戻り値は「/bar/baz/1」
     *   - param(0)の戻り値は「bar」
     *   - param(1)の戻り値は「baz」
     *
     * @access public
     * @param int $index パラメータのインデックス
     * @return string アクションにPATH_INFOで渡されたパラメータ
     */
    function getActionParam($index = null)
    {
        $path_info = $this->getPathInfo();
        $action_path = $this->getActionPath();
        $param = substr(ltrim($path_info, '/'), strlen($action_path));
        $param = ltrim($param, '/');
        if (strlen($param) == 0) {
            $param = null;
        }
        if (is_null($index)) {
            return $param;
        } else {
            if (is_null($param)) {
                return null;
            }
            $params = explode('/', $param);
            if ($index < count($params)) {
                return $params[$index];
            }
        }
    }

    /**
     * アクションパスを設定します。
     *
     * @access public
     * @param string $action_path アクションパス
     */
    function setActionPath($action_path)
    {
        $this->_action_path = $action_path;
    }

    /**
     * アクションパスを取得します。
     *
     * @access public
     * @return string アクションパス
     */
    function getActionPath()
    {
        return $this->_action_path;
    }

    /**
     * アクションのURLの基準となるURLを取得します。
     * サーバがAapcheの場合、$_SERVER['REQUEST_URI']の末尾から
     * QUERY_STRINGとPATH_INFOを除去した値を返します。
     * サーバがIISの場合、$_SERVER['URL']の末尾から
     * PATH_INFOを除去した値を返します。
     * リクエストURLが「http://example.com/test/index.php/foo」の場合、
     * 「/test/index.php」を返します。mod_rewriteが使用されていて
     * 「http://example.com/test/foo」となっている場合、「/test」を返します。
     * このメソッドはMoony_Action::redirectAction()で使用されます。
     *
     * @access public
     * @return string アクションの基準となるURL
     */
    function getActionBase()
    {
        $request_uri = null;
        if (isset($_SERVER['REQUEST_URI'])) {
            // Apache
            $request_uri = $this->getServer('REQUEST_URI');
            if (($pos = strpos($request_uri, '?')) !== false) {
                $request_uri = substr($request_uri, 0, $pos);
            }
        } else {
            // IIS
            $request_uri = $this->getServer('URL');
        }
        $path_info = $this->getPathInfo();
        return substr($request_uri, 0,
            strlen($request_uri) - strlen($path_info));
    }

    /**
     * エンコーディングの変換を行います。
     *
     * @access public
     * @param string $from_encoding 変換元エンコーディング
     * @param string $to_encoding 変換先エンコーディング
     */
    function convertEncoding($from_encoding, $to_encoding)
    {
        mb_convert_variables($to_encoding, $from_encoding,
            $this->_params, $this->_files, $this->_query_string);
    }

    /**
     * リクエストパラメータのフィルタ処理を行います。
     * magic_quotes_gpcがONの場合、不要なスラッシュを除去します。
     * また、null文字が含まれている場合、除去します。
     *
     * @access protected
     * @param string|array $var フィルタ対象の値
     * @return string|array フィルタ済みの値
     */
    function _filter($var)
    {
        static $magic_quotes_gpc;
        if (is_null($magic_quotes_gpc)) {
            $magic_quotes_gpc = get_magic_quotes_gpc();
        }
        if (is_array($var)) {
            return array_map(array($this, '_filter'), $var);
        }
        $var = str_replace("\0", '', $var);
        if ($magic_quotes_gpc) {
            $var = stripslashes($var);
        }
        return $var;
    }
}
?>
